/*
 * JarSearchWorker.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jarsearch;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;
import java.util.jar.JarFile;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;

import org.underworldlabs.util.MiscUtils;

/** 
 * JAR file search worker.
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1333 $
 * @date     $Date: 2008-06-24 21:19:51 +1000 (Tue, 24 Jun 2008) $
 */
public class JarSearchWorker {

    /** jar file extension */
//    private static final String JAR_EXTENSION = ".jar";
    
    /** valid file extensions */
    private static final String VALID_EXTENSIONS = "jar|war|ear";
    
    /** the controller object */
    private JarSearchController controller;
    
    /** the jar files searched count */
    private int jarFileCount;
    
    /** the number of jars found */
    private int findCount;
    
    private boolean searchResources;
    
    /** Creates a new instance of JarSearchWorker */
    public JarSearchWorker(JarSearchController controller) {
        this.controller = controller;
    }
    
    public void search(String namePattern, Vector<String> paths, 
            boolean matchCase, boolean useRegex, boolean wholeWords, boolean searchResources) 
    throws InterruptedException, MalformedURLException, IOException {
        
        this.searchResources = searchResources;
        
        String regexPattern = null;
        if (useRegex) {
            
            if (namePattern.startsWith("*.")) {
                
                namePattern = namePattern.replace("*.", ".*\\.");
                
            }
            regexPattern = namePattern;
            
        } else if (wholeWords) {

            regexPattern = "\\b" + namePattern.replaceAll("\\.", "\\.") + "\\b";

        } else {

            regexPattern = namePattern.replaceAll("\\.", "\\\\.");
        }
        
        regexPattern = replaceWildCards(regexPattern);

        if (matchCase) {
            pattern = Pattern.compile(regexPattern);
        } else {
            pattern = Pattern.compile(regexPattern, Pattern.CASE_INSENSITIVE);
        }
        matcher = pattern.matcher("");
        
        jarFileCount = 0;
        findCount = 0;
        
        for (int i = 0, n = paths.size(); i < n; i++) {
   
            if (Thread.interrupted()) {
                throw new InterruptedException();
            }

            searchPath(namePattern, paths.get(i));
        }
        
    }

    private String replaceWildCards(String regexPattern) {

        StringBuffer sb = new StringBuffer();
        char[] chars = regexPattern.toCharArray();
        for (int i = 0; i < chars.length; i++) {
            
            if (chars[i] == '*') {

                if ((i > 0 && chars[i - 1] != '.') || i == 0) {

                    sb.append('.');
                }

            }
            
            sb.append(chars[i]);            
        }

        return sb.toString();
    }
    
    private Pattern pattern;
    private Matcher matcher;
    
    private void searchJar(String clazzName, String jar) 
        throws InterruptedException, MalformedURLException, IOException {

        jarFileCount++;
        URL[] urls = new URL[1];

        File f = new File(jar);
        urls[0] = f.toURI().toURL();
        
        controller.appendProgress("Scanning JAR file: " + jar);

        JarFile jarFile = null;

        try {

            jarFile = new JarFile(jar);
            
        } catch (ZipException e) {
            
            controller.appendProgress(
                    "Error scanning JAR file " + jar + " - " +e.getMessage() + "\n");
            return;
        }

        String searchName = null;
        for (Enumeration<?> j = jarFile.entries(); j.hasMoreElements();) {

            if (Thread.interrupted()) {
                throw new InterruptedException();
            }

            ZipEntry entry = (ZipEntry)j.nextElement();
            String entryName = entry.getName();

            if (!searchResources || entryName.endsWith(".class")) {

                searchName = MiscUtils.getClassName(entryName);
                if (searchName == null) {

                    continue;
                }

            } else {

                searchName = entryName;
            }

            matcher.reset(searchName);
            //if (className.contains(clazzName)) {
            if (matcher.find()) {
                findCount++;
                controller.addSearchResult(
                        new JarSearchResult(searchName, jar));
            }

        }
    }
    
    private boolean isValidArchiveFileName(String name) {        
        String extension = name.substring(name.lastIndexOf(".") + 1);
        return VALID_EXTENSIONS.contains(extension);
    }
    
    private void searchPath(String clazzName, String path) 
    throws InterruptedException, MalformedURLException, IOException {

        File file = new File(path);
        
        if (file.isFile()) {
        
            if (isValidArchiveFileName(file.getName())) {
                searchJar(clazzName, file.getAbsolutePath());
            }
            return;
        }

        File[] files = file.listFiles();
        if (files != null && files.length > 0) {
            for (int i = 0; i < files.length; i++) {

                if (Thread.interrupted()) {
                    throw new InterruptedException();
                }

                File _file = files[i];

                if (_file.isDirectory()) {
                    searchPath(clazzName, _file.getAbsolutePath());
                }

                if (isValidArchiveFileName(_file.getName())) {
                    searchJar(clazzName, _file.getAbsolutePath());
                }

            }
        }

        
    }

    /**
     * Returns the jar file scan count.
     */
    public int getJarFileCount() {
        return jarFileCount;
    }

    /**
     * Returns the jar file found count.
     */
    public int getFindCount() {
        return findCount;
    }
    
}
