/*
 * SystemResources.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jarsearch;

import  java.io.File;
import java.io.IOException;
import java.util.Properties;
import java.util.Vector;
import javax.swing.JOptionPane;
import org.underworldlabs.util.FileUtils;

/**
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 974 $
 * @date     $Date: 2007-07-24 22:35:32 +1000 (Tue, 24 Jul 2007) $
 */
public class SystemResources {
    
    /**
     * Creates a new instance of SystemResources
     */
    private SystemResources() {}
    
    /**
     * Returns the value of the system property user.home - typically
     * this will be /home/<username> or similar.
     */
    public static String getUserHomeDirectory() {
        return System.getProperty("user.home");
    }
    
    /**
     * Returns the value of the hidden home directory for the current
     * user's settings for this application.
     */
    public static String getUserSettingsHomeDirectory() {
        StringBuffer sb = new StringBuffer();
        sb.append(getUserHomeDirectory());
        sb.append(System.getProperty("file.separator"));
        sb.append(".underworldlabs");
        sb.append(System.getProperty("file.separator"));
        sb.append("jarsearch");
        sb.append(System.getProperty("file.separator"));
        return sb.toString();
    }    
    
    /**
     * Saves the specified properties as the user properties 
     * in the user's application home directory.
     *
     * @param properties - the new user properties
     */
    public static synchronized void saveUserProperties(Properties properties) {
        try {
            FileUtils.storeProperties(getUserSettingsFilePath(),
                                      properties,
                                      "JarClassSearch user defined properties");
        }
        catch (IOException e) {
            JOptionPane.showMessageDialog(
                            null,
                            "Error saving user defined properties", 
                            "Error",
                            JOptionPane.ERROR_MESSAGE);
        }
    }
    
    /**
     * Adds the specified string to the search history.
     *
     * @param value the search string
     */
    public static void addSearchQuery(String value) {
        Vector<String> history = getSearchQueryHistory();
        try {
            // make sure we don't already have it
            for (int i = 0, n = history.size(); i < n; i++) {
                String query = history.get(i);
                if (query.equals(value)) {
                    history.remove(i);
                    history.add(0, query);
                    return;
                }
            }
            
            int size = history.size();
            // allow only last 20 values
            if (size >= 20) {
                history.remove(size - 1);
            }
            history.add(0, value);

        }
        finally {
            saveSearchQueryHistory(history);
        }
    }
    
    /**
     * Saves the search query history.
     */
    public static synchronized void saveSearchQueryHistory(Vector<String> history) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0, n = history.size(); i < n; i++) {
            sb.append(history.get(i));
            if (i < (n - 1)) {
                sb.append("\n");
            }
        }
        try {
            FileUtils.writeFile(
                    getSearchQueryHistoryPath(), sb.toString(), false);
        }
        catch (IOException e) {} // not fussed really
    }
    
    /**
     * Returns the search history.
     */
    public static Vector<String> getSearchQueryHistory() {
        String path = getSearchQueryHistoryPath();
        File file = new File(path);
        if (file.exists()) {
            try {
                String history = FileUtils.loadFile(file);
                String[] values = history.split("\n");
                Vector<String> _history = new Vector<String>(values.length);
                for (int i = 0; i < values.length; i++) {
                    _history.add(values[i]);
                }
                return _history;
            }
            catch (IOException e) {
                return new Vector<String>(0);
            }
        }
        return new Vector<String>(0);
    }
    
    /**
     * Returns the search query history file path.
     */
    public static String getSearchQueryHistoryPath() {
        return getUserSettingsHomeDirectory() + 
                        System.getProperty("file.separator") + 
                        "search.history";
    }

    /**
     * Adds the specified string to the search history.
     *
     * @param value the search string
     */
    public static void addSearchPath(String value) {
        Vector<String> history = getSearchPathHistory();
        try {
            // make sure we don't already have it
            for (int i = 0, n = history.size(); i < n; i++) {
                String query = history.get(i);
                if (query.equals(value)) {
                    history.remove(i);
                    history.add(0, query);
                    return;
                }
            }
            
            int size = history.size();
            // allow only last 20 values
            if (size >= 20) {
                history.remove(size - 1);
            }
            history.add(0, value);

        }
        finally {
            saveSearchPathHistory(history);
        }
    }

    /**
     * Saves the search query history.
     */
    public static synchronized void saveSearchPathHistory(Vector<String> history) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0, n = history.size(); i < n; i++) {
            sb.append(history.get(i));
            if (i < (n - 1)) {
                sb.append("\n");
            }
        }
        try {
            FileUtils.writeFile(
                    getSearchPathHistoryPath(), sb.toString(), false);
        }
        catch (IOException e) {} // not fussed really
    }
    
    /**
     * Returns the search path history.
     */
    public static Vector<String> getSearchPathHistory() {
        String path = getSearchPathHistoryPath();
        File file = new File(path);
        if (file.exists()) {
            try {
                String history = FileUtils.loadFile(file);
                String[] values = history.split("\n");
                Vector<String> _history = new Vector<String>(values.length);
                for (int i = 0; i < values.length; i++) {
                    _history.add(values[i]);
                }
                return _history;
            }
            catch (IOException e) {
                return new Vector<String>(0);
            }
        }
        return new Vector<String>(0);
    }

    /**
     * Returns the search path history file path.
     */
    public static String getSearchPathHistoryPath() {
        return getUserSettingsHomeDirectory() + 
                        System.getProperty("file.separator") + 
                        "path.history";
    }

    public static String getUserSettingsFilePath() {
        return getUserSettingsHomeDirectory() + 
                        System.getProperty("file.separator") + 
                        "user.properties";
    }
    
    /**
     * Returns the application temp directory.
     */
    public static String getTempDirectoryPath() {
        String path = getUserSettingsHomeDirectory() + "temp";
        File file = new File(path);
        if (!file.exists()) {
            file.mkdir();
        }
        return path;
    }
    
    /**
     * Returns the resource path to the default system properties.
     */
    public static String getDefaultPropertiesResourcePath() {
        return "org/underworldlabs/jarsearch/default.properties";
    }

    public static Properties getDefaultProperties() {
        Properties properties = null;
        try {
            String path = getDefaultPropertiesResourcePath();
            properties = FileUtils.loadPropertiesResource(path);
        }
        catch (IOException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(
                            null,
                            "Error opening default system properties", "Error",
                            JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
        return properties;
    }

    /**
     * Checks if the user appalication home dir exists.
     * If it does not, it is created and populated with default
     * properties objects and directories.
     */
    public static void buildUserProperties() {
        try {
            // make sure the settings dir exists
            File userHomeDir = new File(getUserSettingsFilePath());
            if (!userHomeDir.exists()) {
                if (createUserSettingsHomeDirectory()) {
                    FileUtils.copyResource(
                            getDefaultPropertiesResourcePath(), 
                            userHomeDir.getAbsolutePath());
                    // create the temp directory
                    //getTempDirectoryPath();
                }
                else {
                    JOptionPane.showMessageDialog(null,
                                        "Error creating user application home directory",
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
                }
            }
        }
        catch (IOException e) {
            JOptionPane.showMessageDialog(null,
                "Error creating user application home directory:\n" + e.getMessage(),
                "Error",
                JOptionPane.ERROR_MESSAGE);
        }
    }

    public static Properties getUserProperties() {
        try {
            // retrieve the default properties
            Properties defaults = getDefaultProperties();

            // make sure the settings dir exists
            File userHomeDir = new File(getUserSettingsFilePath());
            if (!userHomeDir.exists()) {
                if (createUserSettingsHomeDirectory()) {
                    FileUtils.copyResource(
                            getDefaultPropertiesResourcePath(), 
                            userHomeDir.getAbsolutePath());
                }
                else {
                    JOptionPane.showMessageDialog(null,
                                        "Error creating user application home directory",
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
                    return null;
                }
            }

            Properties properties = FileUtils.loadProperties(userHomeDir, defaults);
            return properties;
        }
        catch (IOException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(null,
                                "Error opening user system properties", "Error",
                                JOptionPane.ERROR_MESSAGE);
            return null;
        }
    }

    private static boolean createUserSettingsHomeDirectory() {
        File userHomeDir = new File(getUserSettingsHomeDirectory());
        return userHomeDir.mkdirs();
    }
    
}
