/*
 * GUIUtilities.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jarsearch.gui;

import java.awt.Dimension;
import java.awt.Point;

import javax.swing.ImageIcon;
import javax.swing.JFrame;

import org.underworldlabs.swing.ExceptionErrorDialog;
import org.underworldlabs.swing.GUIUtils;
import org.underworldlabs.swing.util.IconUtilities;
import org.underworldlabs.util.StringBundle;

/** 
 * Misc helper utils.
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1333 $
 * @date     $Date: 2008-06-24 21:19:51 +1000 (Tue, 24 Jun 2008) $
 */
public class GUIUtilities {
    
    /** the application base panel */
    private static BasePanel basePanel;
    
    /** the status bar panel */
    private static StatusBarPanel statusBar;

    /** the main menu */
    private static MainMenu mainMenu;
    
    /** main application frame */
    private static JFrame frame;
    
    /** icon resource folder path */
    private static final String ICON_FOLDER = 
            "/org/underworldlabs/jarsearch/icons/";
    
    /** string resource loader */
    private static StringBundle bundle;
    
    /** Creates a new instance of GUIUtilities */
    private GUIUtilities() {}
    
    /**
     * Returns the string resource value with the specified name.
     *
     * @param name  the property key name
     * @return the resource string
     */
    public static String getResourceString(String name) {
        return getBundle().getString(name);
    }
    
    /**
     * Returns the resource bundle for the gui package.
     */
    public static StringBundle getBundle() {
        if (bundle == null) {
            bundle = StringBundle.loadBundle(GUIUtilities.class);
        }
        return bundle;
    }
    
    /**
     * Returns the application status bar panel.
     */
    public static StatusBarPanel getStatusBarPanel() {
        if (statusBar == null) {
            statusBar = new StatusBarPanel();
            statusBar.setThirdLabelText(
                    "JDK" + System.getProperty("java.version").substring(0,5));
        }
        return statusBar;
    }

    /**
     * Returns the base panel component.
     */
    public static BasePanel getBasePanel() {
        if (basePanel == null) {
            basePanel = new BasePanel();
        }
        return basePanel;
    }

    /**
     * Returns the main application menu bar.
     */
    public static MainMenu getMainMenu() {
        if (mainMenu == null) {
            mainMenu = new MainMenu();
        }
        return mainMenu;
    }

    /**
     * Sets the label text in the first (left-most) label of
     * the status bar.
     *
     * @param text - the text to be set
     */
    public static void setActivityText(String text) {
        statusBar.setFirstLabelText(text);
    }

    /**
     * Sets the label text in the second label of the status bar.
     *
     * @param text - the text to be set
     */
    public static void setStatusText(String text) {
        statusBar.setSecondLabelText(text);
    }

    /**
     * Indicates that a process has begun and kicks off the
     * status bar's indeterminate progress bar.
     */
    public static void startProcessing() {
        statusBar.startProcessing();
    }

    /**
     * Indicates that a process has finished and stops the
     * status bar's indeterminate progress bar.
     */
    public static void stopProcessing() {
        statusBar.stopProcessing();
    }

    /**
     * Returns the application frame.
     */
    public static JFrame getApplicationFrame() {
        return frame;
    }
    
    /** Registers the application frame with this class. */
    public static void registerApplicationFrame(JFrame aFrame) {
        frame = aFrame;
    }
    
    
    /** <p>Calculates and returns the centered position
     *  of a dialog with the specified size to be added
     *  to the desktop area - ie. taking into account the
     *  size and location of all docked panels.
     *
     *  @param the size of the dialog to be added as a
     *         <code>Dimension</code> object
     *  @return the <code>Point</code> at which to add the dialog
     */
    public static Point getLocationForDialog(Dimension dialogDim) {        
        return GUIUtils.getLocationForDialog(
                    getApplicationFrame(), dialogDim);       
    }

    /**
     * Loads the specified icon from the application icon dir.
     *
     * @param fileName  the icon file name
     * @return the loaded icon
     */
    public static ImageIcon loadIcon(String fileName) {
        return IconUtilities.loadIcon(ICON_FOLDER + fileName);
    }

    /** 
     * Sets the application cursor to the system normal cursor 
     */
    public static void showNormalCursor() {
        GUIUtils.showNormalCursor(getApplicationFrame());
    }
    
    /** 
     * Sets the application cursor to the system wait cursor 
     */
    public static void showWaitCursor() {
        GUIUtils.showWaitCursor(getApplicationFrame());
    }

    /**
     * Displays the error dialog displaying the stack trace from a
     * throws/caught exception.
     *
     * @param message - the error message to display
     * @param e - the throwable
     */
    public static void displayExceptionErrorDialog(String message, Throwable e) {
        new ExceptionErrorDialog(getApplicationFrame(), message, e);
    }

    // -------------------------------------------------------
    // ------ Helper methods for various option dialogs ------
    // -------------------------------------------------------
    
    // These have been revised to use JDialog as the wrapper to
    // ensure the dialog is centered within the dektop pane and not
    // within the entire screen as you get with JOptionPane.showXXX()
    
    public static final void displayInformationMessage(String message) {
        GUIUtils.displayInformationMessage(getApplicationFrame(), message);
    }
    
    public static final String displayInputMessage(String title, String message) {
        return GUIUtils.displayInputMessage(getApplicationFrame(), title, message);
    }
    
    public static final void displayWarningMessage(String message) {
        GUIUtils.displayWarningMessage(getApplicationFrame(), message);
    }
    
    public static final void displayErrorMessage(String message) {
        GUIUtils.displayErrorMessage(getApplicationFrame(), message);
    }

    public static final int displayConfirmCancelErrorMessage(String message) {
        return GUIUtils.displayConfirmCancelErrorMessage(getApplicationFrame(), message);
    }

    public static final int displayYesNoDialog(String message, String title) {
        return GUIUtils.displayYesNoDialog(getApplicationFrame(), message, title);
    }
    
    public static final int displayConfirmCancelDialog(String message) {
        return GUIUtils.displayConfirmCancelDialog(getApplicationFrame(), message);
    }
    
    public static final int displayConfirmDialog(String message) {
        return GUIUtils.displayConfirmDialog(getApplicationFrame(), message);
    }


}





