/*
 * SearchResultsTable.java
 *
 * Copyright (C) 2007 Takis Diakoumis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

package org.underworldlabs.jarsearch.gui;

import java.awt.Component;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumnModel;

import org.underworldlabs.jarsearch.JarSearchResult;
import org.underworldlabs.swing.GUIUtils;
import org.underworldlabs.swing.table.AbstractSortableTableModel;
import org.underworldlabs.swing.table.TableSorter;

/** 
 * Search results table display using a custom header renderer.
 *
 * @author   Takis Diakoumis
 * @version  $Revision: 1333 $
 * @date     $Date: 2008-06-24 21:19:51 +1000 (Tue, 24 Jun 2008) $
 */
public class SearchResultsTable extends JTable {
    
    /** the table model */
    private SearchResultsTableModel model;
    
    /** Creates a new instance of DefaultTable */
    public SearchResultsTable() {

        model = new SearchResultsTableModel();
        TableSorter sorter = new TableSorter(model);
        setModel(sorter);
        sorter.setTableHeader(getTableHeader());

        /*
        //if (UIUtils.isDefaultLookAndFeel() || UIUtils.usingOcean()) {
        if (UIManager.getLookAndFeel() instanceof MetalLookAndFeel) {
            getTableHeader().setDefaultRenderer(new DefaultTableHeaderRenderer());
        }
         */

        setDefaultRenderer(String.class, new SearchResultTableCellRenderer());
        setRowHeight(20);
        setDragEnabled(true);
        
        getTableHeader().setReorderingAllowed(false);
        addComponentListener(new SearchResultsTableAdapter());
        
        new SearchResultsPopupMenu(this);
    }
    
    /**
     * Clears the table of all data.
     */
    public void clear() {
        model.clear();
    }
    
    /**
     * Adds the specified item to the results table.
     *
     * @param item   the result to be added
     */
    public void addItem(JarSearchResult item) {
        model.addItem(item);
    }
    
    class SearchResultsTableAdapter extends ComponentAdapter {
        private boolean resizeOff;
        public void componentResized(ComponentEvent e) {
            if (!resizeOff) {
                GUIUtils.invokeLater(new Runnable() {
                    public void run() {
                        resizeOff = true;
                        setAutoResizeMode(AUTO_RESIZE_OFF);
                        int columnWidth = getWidth() / 2;
                        TableColumnModel tcm = getColumnModel();
                        tcm.getColumn(0).setPreferredWidth(columnWidth);
                        tcm.getColumn(1).setPreferredWidth(columnWidth);
                    }
                });
            }
        }
    }
            
    class SearchResultsTableModel extends AbstractSortableTableModel {
        
        /** table header labels */
        private String[] header = {"Name", 
                                   "JAR File"};

        private Vector<JarSearchResult> searchResults;

        /**
         * Clears the table of all data.
         */
        public void clear() {
            if (searchResults == null) {
                return;
            }
            searchResults.clear();
            fireTableDataChanged();
        }

        /**
         * Adds the specified item to the results table.
         *
         * @param item   the result to be added
         */
        public void addItem(JarSearchResult item) {
            if (searchResults == null) {
                searchResults = new Vector<JarSearchResult>();
            }
            searchResults.add(item);
            fireTableDataChanged();
        }

        /** Returns the table column count */
        public int getColumnCount() {
            return header.length;
        }
        
        /** Returns the table row count */
        public int getRowCount() {
            if (searchResults == null) {
                return 0;
            }
            return searchResults.size();
        }

        public Class<?> getColumnClass(int columnIndex) {
            return String.class;
        }

        public boolean isCellEditable(int row, int col) {
            return true;
        }

        public String getColumnName(int col) {
            return header[col];
        }

        public Object getValueAt(int row, int col) {
            JarSearchResult result = searchResults.get(row);
            if (col == 0) {
                return result.getClazzName();
            } else if (col == 1) {
                return result.getJarFile();
            }
            return null;
        }
        
    }
    
    
    class SearchResultTableCellRenderer extends DefaultTableCellRenderer {
        private ImageIcon nameIcon;
        private ImageIcon jarIcon;
        public Component getTableCellRendererComponent(JTable table,
                                                       Object value, 
                                                       boolean isSelected, 
                                                       boolean hasFocus,
                                                       int row, int col) {
        
            if (value != null) {
                setToolTipText(value.toString());
            }

            if (col == 0) {
                if (nameIcon == null) {
                    nameIcon = GUIUtilities.loadIcon("class.gif");
                }
                setIcon(nameIcon);
            }
            else if (col == 1) {
                if (jarIcon == null) {
                    jarIcon = GUIUtilities.loadIcon("Jar16.gif");
                }
                setIcon(jarIcon);
            }
            
            return super.getTableCellRendererComponent(
                    table, value, isSelected, hasFocus, row, col);

        }
    }
    
    private static final long serialVersionUID = 1L;
}

